/*
 * Copyright (c) 2004 by Sun Microsystems, Inc.
 * All rights reserved.
 */

#pragma ident	"@(#)ixge_mac.h 1.13	05/08/15 SMI"

#ifndef	_IXGE_MAC_H
#define	_IXGE_MAC_H

#ifdef	__cplusplus
extern "C" {
#endif

/*
 * This number is the subsystem device ID.
 * It uniquely determines the board revision.
 */
#define	IXGE_SUBSYSTEM_REV_P0	0xa11f
#define	IXGE_SUBSYSTEM_REV_P1	0x7036

#define	IXGE_MAX_HEADER 250
#define RX_CSUM_OFFSET 34


/* Hardware reset */
typedef enum  {
	IXGE_TX_DISABLE,		/* Disable Tx side */
	IXGE_RX_DISABLE,		/* Disable Rx side */
	IXGE_CHIP_RESET			/* Full chip reset */
} ixge_reset_t;

#define IXGE_DELAY_AFTER_TXRX   10000    /* 10ms after idling rx/tx units */
#define IXGE_DELAY_AFTER_MAC_RESET  2000      /* 2ms after h/w reset */
#define IXGE_DELAY_AFTER_XFP_RESET  305000      /* 305ms after the reset */
	/* Changed to 200ms from 10ms based on Bug 2504 */
#define IXGE_DELAY_AFTER_EE_RESET 200000  /* 200ms after the EEPROM reset */
#define IXGE_DELAY_AFTER_LINK_RESET 13  /* 13 Us after link reset */
#define IXGE_LINK_RESETS 8	/* Maxiumum PHY resets to wait for link-up */

/*
 * TxIntDelay Valid Range: 0-65535 (0=off) Default Value: 32 This value
 * delays the generation of transmit interrupts in units of 1.024
 * microseconds. Transmit interrupt reduction can improve CPU efficiency if
 * properly tuned for specific network traffic. If the system is reporting
 * dropped transmits, this value may be set too high causing the driver to
 * run out of available transmit descriptors.
 */
#define IXGE_TX_INTR_DELAY 32



/*
 * Need to override the compiler structure packing
 * since the hardware is not compatible with the compiler 
 * packing.
 */
#pragma pack(2)

#define IXGE_MAX_FLOWS	64

typedef struct _pci_cfg_t {
	uint16_t vendorid;
	uint16_t devid;
	uint16_t command;
	uint16_t status;
	uint8_t  revid;
	uint8_t  res0;
	uint16_t junk1;
	uint8_t  cache_line;
	uint8_t  latency;
	uint8_t  header;
	uint8_t  bist;
	uint32_t base;
	uint32_t base14;
	uint32_t base18;
	uint32_t base1c;
	uint32_t base20;
	uint32_t base24;
	uint32_t base28;
	uint32_t base2c;
	uint32_t base30;
	uint32_t res1[2];
	uint8_t int_line;
	uint8_t int_pin;
	uint8_t	min_gnt;
	uint8_t max_lat;
} pci_cfg_t, *p_pci_cfg_t;


/* ======================================= */

/*
 * General Registers
 */

/* CTRL0 - Device Control Register 0 */
typedef union _ctrl0_t {
	uint32_t val;
	struct {
#ifdef _BIT_FIELDS_HTOL
		uint32_t res3:1;
		uint32_t vme:1;
		uint32_t res2:1;
		uint32_t tpe:1;
		uint32_t rpe:1;
		uint32_t rst:1;
		uint32_t swdpiolo3:1;
		uint32_t swdpiolo2:1;
		uint32_t swdpiolo1:1;
		uint32_t swdpiolo0:1;
		uint32_t swdpinslo3:1;
		uint32_t swdpinslo2:1;
		uint32_t swdpinslo1:1;
		uint32_t swdpinslo0:1;
		uint32_t res1:10;
		uint32_t cmdc:1;
		uint32_t mdcs:1;
		uint32_t xle:1;
		uint32_t jfe:1;
		uint32_t lrst:1;
		uint32_t res0:3;
#else
		uint32_t res0:3;
		uint32_t lrst:1;
		uint32_t jfe:1;
		uint32_t xle:1;
		uint32_t mdcs:1;
		uint32_t cmdc:1;
		uint32_t res1:10;
		uint32_t swdpinslo0:1;
		uint32_t swdpinslo1:1;
		uint32_t swdpinslo2:1;
		uint32_t swdpinslo3:1;
		uint32_t swdpiolo0:1;
		uint32_t swdpiolo1:1;
		uint32_t swdpiolo2:1;
		uint32_t swdpiolo3:1;
		uint32_t rst:1;
		uint32_t rpe:1;
		uint32_t tpe:1;
		uint32_t res2:1;
		uint32_t vme:1;
		uint32_t res3:1;
#endif
	} bits;
} ctrl0_t, *p_ctrl0_t;

/* CTRL1 - Device Control Register 1 */
typedef union _ctrl1_t {
	uint32_t val;
	struct {
#ifdef _BIT_FIELDS_HTOL
		uint32_t res2:12;
		uint32_t pcixhm:2;
		uint32_t ro_dis:1;
		uint32_t res1:3;
		uint32_t ee_rst:1;
		uint32_t res0:1;
		uint32_t swdpiohi3:1;
		uint32_t swdpiohi2:1;
		uint32_t swdpiohi1:1;
		uint32_t swdpiohi0:1;
		uint32_t swdpinshi3:1;
		uint32_t swdpinshi2:1;
		uint32_t swdpinshi1:1;
		uint32_t swdpinshi0:1;
		uint32_t gpi_en:4;
#else
		uint32_t gpi_en:4;
		uint32_t swdpinshi0:1;
		uint32_t swdpinshi1:1;
		uint32_t swdpinshi2:1;
		uint32_t swdpinshi3:1;
		uint32_t swdpiohi0:1;
		uint32_t swdpiohi1:1;
		uint32_t swdpiohi2:1;
		uint32_t swdpiohi3:1;
		uint32_t res0:1;
		uint32_t ee_rst:1;
		uint32_t res1:3;
		uint32_t ro_dis:1;
		uint32_t pcixhm:2;
		uint32_t res2:12;
#endif
	} bits;
} ctrl1_t, *p_ctrl1_t;

/* STATUS - Device Status Register */
typedef union _status_t {
	uint32_t val;
	struct {
#ifdef _BIT_FIELDS_HTOL
		uint32_t res4:12;
		uint32_t revid:4;
		uint32_t pcix_spd:2;
		uint32_t pcix_mode:1;
		uint32_t bus64:1;
		uint32_t pci_spd:1;
		uint32_t res3:5;
		uint32_t xauime:1;
		uint32_t txoff:1;
		uint32_t res2:1;
		uint32_t aip:1;
		uint32_t lu:1;
		uint32_t res1:1;
#else
		uint32_t res1:1;
		uint32_t lu:1;
		uint32_t aip:1;
		uint32_t res2:1;
		uint32_t txoff:1;
		uint32_t xauime:1;
		uint32_t res3:5;
		uint32_t pci_spd:1;
		uint32_t bus64:1;
		uint32_t pcix_mode:1;
		uint32_t pcix_spd:2;
		uint32_t revid:4;
		uint32_t res4:12;
#endif
	} bits;
} status_t, *p_status_t;


#define	IXGE_EEPROM_SETTLE_TIME	2	/* 1 Us is the minimum, so use 2Us */
#define IXGE_EEPROM_CHECKSUM 0xBABA

/* EEPROM structure */
typedef struct _ixge_eeprom_map_t {
	struct ether_addr macaddr;
	uint16_t	compatibility;
	uint16_t	res1[4];
	uint32_t	pba;
	uint16_t	init_ctrl1;
	uint16_t	subsystem_id;
	uint16_t	subsystem_vendor;
	uint16_t	device_id;
	uint16_t	vendor_id;
	uint16_t	init_ctrl2;
	uint16_t	res2[16];
	uint16_t	swdpins_reg;
	uint16_t	circuit_ctrl_reg;
	uint8_t		d0_power;
	uint8_t		d3_power;
	uint16_t	res3[28];
	uint16_t	checksum;
} ixge_eeprom_map_t, *p_ixge_eeprom_map_t;

/* Hardware NIC type */
typedef enum {
	IXGE_KEALIA_NIC	= 0,
	IXGE_INTEL_NIC,
	IXGE_UNKNOWN_NIC
} ixge_boardtype_t;

/* EECD - EEPROM/FLASH Control & Data Register */
typedef union _eecd_t {
	uint32_t val;
	struct {
#ifdef _BIT_FIELDS_HTOL
		uint32_t res0:26;
		uint32_t few:2;
		uint32_t ee_do:1;
		uint32_t ee_di:1;
		uint32_t ee_cs:1;
		uint32_t ee_sk:1;
#else
		uint32_t ee_sk:1;
		uint32_t ee_cs:1;
		uint32_t ee_di:1;
		uint32_t ee_do:1;
		uint32_t few:2;
		uint32_t res0:26;
#endif
	} bits;
} eecd_t, *p_eecd_t;

/* MFS - Max Frame Size */
typedef union _mfs_t {
	uint32_t val;
	struct {
#ifdef _BIT_FIELDS_HTOL
		uint32_t max_frame_size:16;
		uint32_t res0:16;
#else
		uint32_t res0:16;
		uint32_t max_frame_size:16;
#endif
	} bits;
} mfs_t, *p_mfs_t;


/* ======================================= */

/*
 * Interrupt Registers
 *
 * ICR - Interrupt Cause Read Register
 * ICS - Interrupt Cause Set Register
 * IMS - Interrupt Mask Set/Read Register
 * IMC - Interrupt Mask Clear Register
 */
typedef union _intr_t {
	uint32_t val;
	struct {
#ifdef _BIT_FIELDS_HTOL
		uint32_t res3:17;
		uint32_t gpi:4;
		uint32_t res2:1;
		uint32_t autoscan:1;
		uint32_t res1:1;
		uint32_t rxt0:1;
		uint32_t rxo:1;
		uint32_t res0:1;
		uint32_t rxdmt0:1;
		uint32_t rxseq:1;
		uint32_t lsc:1;
		uint32_t txqe:1;
		uint32_t txdw:1;
#else
		uint32_t txdw:1;
		uint32_t txqe:1;
		uint32_t lsc:1;
		uint32_t rxseq:1;
		uint32_t rxdmt0:1;
		uint32_t res0:1;
		uint32_t rxo:1;
		uint32_t rxt0:1;
		uint32_t res1:1;
		uint32_t autoscan:1;
		uint32_t res2:1;
		uint32_t gpi:4;
		uint32_t res3:17;
#endif
	} bits;
} intr_t, *p_intr_t;

/* ======================================= */

/*
 * Receive Registers
 */

/*
 * Used for determining whether to set/clear rxen/txen
 */
#define	IXGE_DMA_START	B_TRUE
#define	IXGE_DMA_STOP	B_FALSE

/* RCTL - Receive Control Register */
typedef union _rxctl_t {
	uint32_t val;
	struct {
#ifdef _BIT_FIELDS_HTOL
		uint32_t res5:5;
		uint32_t secrc:1;
		uint32_t res4:2;
		uint32_t cff:1;
		uint32_t rpda:2;
		uint32_t cfi:1;
		uint32_t cfien:1;
		uint32_t vfe:1;
		uint32_t bsize:2;
		uint32_t bam:1;
		uint32_t res3:1;
		uint32_t mo:2;
		uint32_t res2:2;
		uint32_t rdmts:2;
		uint32_t res1:3;
		uint32_t mpe:1;
		uint32_t upe:1;
		uint32_t sbp:1;
		uint32_t rxen:1;
		uint32_t res0:1;
#else
		uint32_t res0:1;
		uint32_t rxen:1;
		uint32_t sbp:1;
		uint32_t upe:1;
		uint32_t mpe:1;
		uint32_t res1:3;
		uint32_t rdmts:2;
		uint32_t res2:2;
		uint32_t mo:2;
		uint32_t res3:1;
		uint32_t bam:1;
		uint32_t bsize:2;
		uint32_t vfe:1;
		uint32_t cfien:1;
		uint32_t cfi:1;
		uint32_t rpda:2;
		uint32_t cff:1;
		uint32_t res4:2;
		uint32_t secrc:1;
		uint32_t res5:5;
#endif
	} bits;
} rxctl_t, *p_rxctl_t;

/* FCRTL/FCRTH - Flow Control Receive Threshold Low/High */
typedef union _fcrt_t {
	uint32_t val;
	struct {
#ifdef _BIT_FIELDS_HTOL
		uint32_t xone:1;
		uint32_t res1:13;
		uint32_t rt:15;
		uint32_t res0:3;
#else
		uint32_t res0:3;
		uint32_t rt:15;
		uint32_t res1:13;
		uint32_t xone:1;
#endif
	} bits;
} fcrt_t, *p_fcrt_t;

/* RDTR - Receive Delay Timer Register */
typedef union _rdtr_t {
	uint32_t val;
	struct {
#ifdef _BIT_FIELDS_HTOL
		uint32_t fpd:1;
		uint32_t res0:15;
		uint32_t delay:16;
#else
		uint32_t delay:16;
		uint32_t res0:15;
		uint32_t fpd:1;
#endif
	} bits;
} rdtr_t, *p_rdtr_t;

/* RXDCTL - Receive Descriptor Control */
typedef union _rxdctl_t {
	uint32_t val;
	struct {
#ifdef _BIT_FIELDS_HTOL
		uint32_t res0:5;
		uint32_t wthresh:9;
		uint32_t hthresh:9;
		uint32_t pthresh:9;
#else
		uint32_t pthresh:9;
		uint32_t hthresh:9;
		uint32_t wthresh:9;
		uint32_t res0:5;
#endif
	} bits;
} rxdctl_t, *p_rxdctl_t;

/* RAIDC - Receive Adaptive Interrupt Delay Control */
typedef union _raidc_t {
	uint32_t val;
	struct {
#ifdef _BIT_FIELDS_HTOL
		uint32_t en:1;
		uint32_t rxt_gate:1;
		uint32_t res1:1;
		uint32_t poll:9;
		uint32_t delay:9;
		uint32_t res0:5;
		uint32_t highthrs:6;
#else
		uint32_t highthrs:6;
		uint32_t res0:5;
		uint32_t delay:9;
		uint32_t poll:9;
		uint32_t res1:1;
		uint32_t rxt_gate:1;
		uint32_t en:1;
#endif
	} bits;
} raidc_t, *p_raidc_t;

/* RXCSUM - Receive Checksum Control */
typedef union _rxcsum_t {
	uint32_t val;
	struct {
#ifdef _BIT_FIELDS_HTOL
		uint32_t res1:22;
		uint32_t tuofl:1;
		uint32_t ipofl:1;
		uint32_t res0:8;
#else
		uint32_t res0:8;
		uint32_t ipofl:1;
		uint32_t tuofl:1;
		uint32_t res1:22;
#endif
	} bits;
} rxcsum_t, *p_rxcsum_t;

/* RAL - Receive Address Low */

/* RAH - Receive Address High */
typedef union _rah_t {
	uint32_t val;
	struct {
#ifdef _BIT_FIELDS_HTOL
		uint32_t av:1;
		uint32_t res0:13;
		uint32_t asel:2;
		uint32_t rah:16;
#else
		uint32_t rah:16;
		uint32_t asel:2;
		uint32_t res0:13;
		uint32_t av:1;
#endif
	} bits;
} rah_t, *p_rah_t;

#define	IXGE_MAC_ENTRIES	16

typedef struct _mac_addr {
	uint32_t ral;
	rah_t	rah;
} mac_addr_t, *p_mac_addr_t;

/* RPR - Receive Page Register */
typedef union _rx_rpr_t {
	uint32_t val;
	struct {
#ifdef _BIT_FIELDS_HTOL
		uint32_t res2:11;
		uint32_t msel:1;
		uint32_t res1:2;
		uint32_t page:6;
		uint32_t res0:12;
#else
		uint32_t res0:12;
		uint32_t page:6;
		uint32_t res1:2;
		uint32_t msel:1;
		uint32_t res2:11;
#endif
	} bits;
} rx_rpr_t, *p_rx_rpr_t;

/* XXX MTA - Multicast Table Array */
/* XXX VFTA - VLAN Filter Table Array */

/* ======================================= */

/*
 * Transmit Registers
 */

/* TCTL - Transmit Control Register */
typedef union _tctl_t {
	uint32_t val;
	struct {
#ifdef _BIT_FIELDS_HTOL
		uint32_t res0:29;
		uint32_t tpde:1;
		uint32_t txen:1;
		uint32_t tce:1;
#else
		uint32_t tce:1;
		uint32_t txen:1;
		uint32_t tpde:1;
		uint32_t res0:29;
#endif
	} bits;
} tctl_t, *p_tctl_t;

/* TIDV - Transmit Interrupt Delay Value */
typedef union _tidv_t {
	uint32_t val;
	struct {
#ifdef _BIT_FIELDS_HTOL
		uint32_t res0:16;
		uint32_t tidv:16;
#else
		uint32_t tidv:16;
		uint32_t res0:16;
#endif
	} bits;
} tidv_t, *p_tidv_t;

/* TXDCTL - Transmit Descriptor Control */
typedef union _txdctl_t {
	uint32_t val;
	struct {
#ifdef _BIT_FIELDS_HTOL
		uint32_t res2:9;
		uint32_t wthresh:7;
		uint32_t res1:1;
		uint32_t hthresh:7;
		uint32_t res0:1;
		uint32_t pthresh:7;
#else
		uint32_t pthresh:7;
		uint32_t res0:1;
		uint32_t hthresh:7;
		uint32_t res1:1;
		uint32_t wthresh:7;
		uint32_t res2:9;
#endif
	} bits;
} txdctl_t, *p_txdctl_t;

/* TSPMT - TCP Segmentation Pad and Minimum Threshold */
typedef union _tspmt_t {
	uint32_t val;
	struct {
#ifdef _BIT_FIELDS_HTOL
		uint32_t tspbp:16;
		uint32_t tsmt:16;
#else
		uint32_t tsmt:16;
		uint32_t tspbp:16;
#endif
	} bits;
} tspmt_t, *p_tspmt_t;

/* PAP - Pause and Pace */
typedef union _pap_t {
	uint32_t val;
	struct {
#ifdef _BIT_FIELDS_HTOL
		uint32_t res0:12;
		uint32_t tx_pace_val:4;
		uint32_t tx_pause_cnt:16;
#else
		uint32_t tx_pause_cnt:16;
		uint32_t tx_pace_val:4;
		uint32_t res0:12;
#endif
	} bits;
} pap_t, *p_pap_t;

/* TPR - Transmit Page Register */
typedef union _tx_tpr_t {
	uint32_t val;
	struct {
#ifdef _BIT_FIELDS_HTOL
		uint32_t res2:11;
		uint32_t msel:1;
		uint32_t res1:5;
		uint32_t page:3;
		uint32_t res0:12;
#else
		uint32_t res0:12;
		uint32_t page:3;
		uint32_t res1:5;
		uint32_t msel:1;
		uint32_t res2:11;
#endif
	} bits;
} tx_tpr_t, *p_tx_tpr_t;

/* ======================================= */

/*
 * Physical Registers
 */

/* PCSC - PCS Control 1 */
typedef union _pcsc_t {
	uint32_t val;
	struct {
#ifdef _BIT_FIELDS_HTOL
		uint32_t res0:16;
		uint32_t res1:1;
		uint32_t res2:1;
		uint32_t speed_10g_sel1:1;
		uint32_t res3:6;
		uint32_t speed_10g_sel2:1;
		uint32_t speed_sel:4;
		uint32_t res4:2;
#else
		uint32_t res4:2;
		uint32_t speed_sel:4;
		uint32_t speed_10g_sel2:1;
		uint32_t res3:6;
		uint32_t speed_10g_sel1:1;
		uint32_t res2:1;
		uint32_t res1:1;
		uint32_t res0:16;
#endif
	} bits;
} pcsc_t, *p_pcsc_t;

#define	IXGE_PCSC2_CAP10G_X_PCS	0x01

/* PCSC2 - PCS Control 2 */
typedef union _pcsc2_t {
	uint32_t val;
	struct {
#ifdef _BIT_FIELDS_HTOL
		uint32_t res0:30;
		uint32_t pcs_type_sel:2;
#else
		uint32_t pcs_type_sel:2;
		uint32_t res0:30;
#endif
	} bits;
} pcsc2_t, *p_pcsc2_t;

/* PCSS - PCS Status 1 */
typedef union _pcss_t {
	uint32_t val;
	struct {
#ifdef _BIT_FIELDS_HTOL
		uint32_t res0:16;
		uint32_t res1:8;
		uint32_t local_fault:1;
		uint32_t res2:4;
		uint32_t recv_link_status:1;
		uint32_t res3:2;
#else
		uint32_t res3:2;
		uint32_t recv_link_status:1;
		uint32_t res2:4;
		uint32_t local_fault:1;
		uint32_t res1:8;
		uint32_t res0:16;
#endif
	} bits;
} pcss_t, *p_pcss_t;

#define	IXGE_PCSS2_DEVICE_PRESENT	0x2

/* PCSS2 - PCS Status 2 */
typedef union _pcss2_t {
	uint32_t val;
	struct {
#ifdef _BIT_FIELDS_HTOL
		uint32_t res0:16;
		uint32_t device_present:2;
		uint32_t res1:2;
		uint32_t tx_local_fault:1;
		uint32_t rx_local_fault:1;
		uint32_t res2:7;
		uint32_t cap_10gbase_w:1;
		uint32_t cap_10gbase_x:1;
		uint32_t cap_10gbase_r:1;
#else
		uint32_t cap_10gbase_r:1;
		uint32_t cap_10gbase_x:1;
		uint32_t cap_10gbase_w:1;
		uint32_t res2:7;
		uint32_t rx_local_fault:1;
		uint32_t tx_local_fault:1;
		uint32_t res1:2;
		uint32_t device_present:2;
		uint32_t res0:16;
#endif
	} bits;
} pcss2_t, *p_pcss2_t;

/* XPCSS - 10GBASE-X PCS Status */
typedef union _xpcss_t {
	uint32_t val;
	struct {
#ifdef _BIT_FIELDS_HTOL
		uint32_t tx_unilink_plllock:1;
		uint32_t rx_unilink_plllock:1;
		uint32_t lane3_sig_detect:1;
		uint32_t lane2_sig_detect:1;
		uint32_t lane1_sig_detect:1;
		uint32_t lane0_sig_detect:1;
		uint32_t lane3_comma_cnt4:1;
		uint32_t lane2_comma_cnt4:1;
		uint32_t lane1_comma_cnt4:1;
		uint32_t lane0_comma_cnt4:1;
		uint32_t lane3_inval_code:1;
		uint32_t lane2_inval_code:1;
		uint32_t lane1_inval_code:1;
		uint32_t lane0_inval_code:1;
		uint32_t align_column_cnt4:1;
		uint32_t deskew_err:1;
		uint32_t res0:3;
		uint32_t lane_align_status:1;
		uint32_t patter_testing:1;
		uint32_t res1:7;
		uint32_t lane3_sync:1;
		uint32_t lane2_sync:1;
		uint32_t lane1_sync:1;
		uint32_t lane0_sync:1;
#else
		uint32_t lane0_sync:1;
		uint32_t lane1_sync:1;
		uint32_t lane2_sync:1;
		uint32_t lane3_sync:1;
		uint32_t res1:7;
		uint32_t patter_testing:1;
		uint32_t lane_align_status:1;
		uint32_t res0:3;
		uint32_t deskew_err:1;
		uint32_t align_column_cnt4:1;
		uint32_t lane0_inval_code:1;
		uint32_t lane1_inval_code:1;
		uint32_t lane2_inval_code:1;
		uint32_t lane3_inval_code:1;
		uint32_t lane0_comma_cnt4:1;
		uint32_t lane1_comma_cnt4:1;
		uint32_t lane2_comma_cnt4:1;
		uint32_t lane3_comma_cnt4:1;
		uint32_t lane0_sig_detect:1;
		uint32_t lane1_sig_detect:1;
		uint32_t lane2_sig_detect:1;
		uint32_t lane3_sig_detect:1;
		uint32_t rx_unilink_plllock:1;
		uint32_t tx_unilink_plllock:1;
#endif
	} bits;
} xpcss_t, *p_xpcss_t;

/* UCCR - Unilink Circuit Control Register */
/* XXXPAS UCCR NOT LISTED IN DEVELOPERS MANUAL??? */

/* XPCSTC - 10GBASE-X PCS Test Control */
typedef union _xpcstc_t {
	uint32_t val;
	struct {
#ifdef _BIT_FIELDS_HTOL
		uint32_t res0:10;
		uint32_t bert_trig:1;
		uint32_t bert_sst:1;
		uint32_t bert_psz:2;
		uint32_t res1:15;
		uint32_t test_pattern_en:1;
		uint32_t test_pattern_sel:2;
#else
		uint32_t test_pattern_sel:2;
		uint32_t test_pattern_en:1;
		uint32_t res1:15;
		uint32_t bert_psz:2;
		uint32_t bert_sst:1;
		uint32_t bert_trig:1;
		uint32_t res0:10;
#endif
	} bits;
} xpcstc_t, *p_xpcstc_t;

/* MACA - MDI Autoscan Command and Address */
typedef union _maca_t {
	uint32_t val;
	struct {
#ifdef _BIT_FIELDS_HTOL
		uint32_t res0:1;
		uint32_t autoscan_intr_type:1;
		uint32_t autoscan_intr_level:1;
		uint32_t autoscan_intr_en:1;
		uint32_t autoscan_en:1;
		uint32_t st_code:2;
		uint32_t devtype_regaddr:5;
		uint32_t reg_bit_sel:4;
		uint32_t autoscan_addr:16;
#else
		uint32_t autoscan_addr:16;
		uint32_t reg_bit_sel:4;
		uint32_t devtype_regaddr:5;
		uint32_t st_code:2;
		uint32_t autoscan_en:1;
		uint32_t autoscan_intr_en:1;
		uint32_t autoscan_intr_level:1;
		uint32_t autoscan_intr_type:1;
		uint32_t res0:1;
#endif
	} bits;
} maca_t, *p_maca_t;

/* APAE - Autoscan PHY Address Enable */
/* ARD - Autoscan Read Data */
/* AID - Autoscan Interrupt Status */

/* MSCA Opcode defines */
#define IXGE_MSCA_OPCODE_ADDR_CYCLE	0x0	/* Addr cycle, New Protocol */
#define IXGE_MSCA_OPCODE_WRITE	0x1		/* Write operation */
#define IXGE_MSCA_OPCODE_READ	0x3		/* Read operation */
#define IXGE_MSCA_OPCODE_READ_AUTO_INC	0x2  /* Read Auto Incr, New Protocol */

#define IXGE_ST_CODE_NEW_PROTOCOL	0x0
#define IXGE_ST_CODE_OLD_PROTOCOL	0x1

/* Definitions for the optics devices on the MDIO bus. */
#define IXGE_PHY_ADDRESS	0X0	/* Single PHY, multiple "Devices" */

/* Standard five-bit Device IDs.  See IEEE 802.3ae, clause 45 */
#define MDIO_PMA_PMD_DID	0x01
#define MDIO_WIS_DID		0x02
#define MDIO_PCS_DID		0x03
#define MDIO_XGXS_DID		0x04

/*
 * Standard PMA/PMD registers and bit definitions.
 * Note: This is a very limited set of definitions,
 * only implemented features are defined. 
 */
#define MDIO_PMA_PMD_CR1	0x0000
#define MDIO_PMA_PMD_CR1_RESET	0x8000

#define MDIO_PMA_PMD_XPAK_VENDOR_NAME 0x803A	/* XPAK/XENPAK devices only */

/* Phy Addresses */
#define IXGE_OPTICAL_PHY_ADDR 0x0 /* Optical Module phy address*/
#define IXGE_XAUII_PHY_ADDR   0x1 /* Xauii transceiver phy address*/
#define IXGE_DIAG_PHY_ADDR    0x1F/* Diagnostic Device phy address*/

/* Kealia: LXT12101 related things */
#define IXGE_LXT0_XCVR    0x00		/* Transceiver Address */
#define IXGE_LXT1_XCVR    0x01

#define IXGE_LXT_DAD      0x1f

#define IXGE_LXT_PMAST    0x1001
#define IXGE_LXT_CFG0     0x8000
#define IXGE_LXT_CFG1     0x8001
#define IXGE_LXT_CFG2     0x8002
#define IXGE_LXT_CFG3     0x8003
#define IXGE_LXT_CFG4     0x8004
#define IXGE_LXT_CFG5     0x8005
#define IXGE_LXT_LANECFG0 0x8009
#define IXGE_LXT_GPIO0    0x9000
#define IXGE_LXT_GPIO1    0x9001
#define IXGE_LXT_GPIO2    0x9002
#define IXGE_LXT_GPIO3    0x9003
#define IXGE_LXT_GPIO4    0x9004
#define IXGE_LXT_STOUT0   0x9015
#define IXGE_LXT_STOUT1   0x9016
#define IXGE_LXT_STOUT2   0x9017

/* MSCA - MDI Single Command and Address */
typedef union _msca_t {
	uint32_t val;
	struct {
#ifdef _BIT_FIELDS_HTOL
		uint32_t mdi_progress_en:1;
		uint32_t mdi_cmd:1;
		uint32_t st_code:2;
		uint32_t op_code:2;
		uint32_t phy_addr:5;
		uint32_t devtype_regaddr:5;
		uint32_t mdi_addr:16;
#else
		uint32_t mdi_addr:16;
		uint32_t devtype_regaddr:5;
		uint32_t phy_addr:5;
		uint32_t op_code:2;
		uint32_t st_code:2;
		uint32_t mdi_cmd:1;
		uint32_t mdi_progress_en:1;
#endif
	} bits;
} msca_t, *p_msca_t;

/* MSRWD - MDI Single Read and Write Data */
typedef union _msrwd_t {
	uint32_t val;
	struct {
#ifdef _BIT_FIELDS_HTOL
		uint32_t mdi_read:16;
		uint32_t mdi_write:16;
#else
		uint32_t mdi_write:16;
		uint32_t mdi_read:16;
#endif
	} bits;
} msrwd_t, *p_msrwd_t;

/* ======================================= */

/*
 * Wake Up Registers - 0x808
 *  Not Used with Solaris
 */

/* WUFC - Wake Up Filter Control Register */
/* WUS - Wake Up Status Register */
/* FFLT - Flexible Filter Lenth Table */
/* FFMT - Flexible Filter Mask Table */
/* FFVT - Flexible Filter Value Table */

/* ======================================= */

/*
 * Statistics Registers - 0x2000
 */

typedef struct _hw_stats_t {
	uint64_t tpr;	/* TPR - Total Packets Received */
	uint64_t gprc;	/* GPRC - Good Packets Received Count */
	uint64_t bprc;	/* BPRC - Broadcast Packets Received Count */
	uint64_t mprc;	/* MPRC - Multicast Packets Received Count */
	uint64_t uprc;	/* UPRC - Unicast Packets Received Count */
	uint64_t vprc;	/* VPRC - VLAN Packets Received Count */
	uint64_t jprc;	/* JPRC - Jumbo Packets Received Count */
	uint64_t gorc;	/* GORC - Good Octets Received Count */
	uint64_t tor;	/* TOR - Total Octets Received */
	uint32_t rnbc;	/* RNBC - Receive No Buffers Count */
	uint32_t res0;
	uint32_t ruc;	/* RUC - Receive Undersize Count */
	uint32_t res1;
	uint32_t roc;	/* ROC - Receive Oversize Count */
	uint32_t res2;
	uint32_t rlec;	/* RLEC - Receive Length Error Count */
	uint32_t res3;
	uint32_t crcerrs;	/* CRCERRS - CRC Error Count */
	uint32_t res4;
	uint32_t icbc;	/* ICBC - Illegal Control Byte in Mid-Packet Count */
	uint32_t res5;
	uint32_t ecbc;	/* ECBC - Error Control BYte in Mid-Packet Count */
	uint32_t res6;
	uint32_t mpc;	/* MPC - Missed Packet Count */
	uint32_t res7[31];
	uint64_t tpt;	/* TPT - Total Packets Transmitted */
	uint64_t gptc;	/* GPTC- Good Packets Transmitted Count */
	uint64_t bptc;	/* BPTC - Broadcast Packets Transmitted Count */
	uint64_t mptc;	/* MPTC - Multicast Packets Transmitted Count */
	uint64_t uptc;	/* UPTC - Unicast Packets Transmitted Count */
	uint64_t vptc;	/* VPTC - VLAN Packets Transmitted Count */
	uint64_t jptc;	/* JPTC - Jumbo Packets Transmitted Count */
	uint64_t gotc;	/* GOTC - Good Octets Transmitted Count */
	uint64_t tot;	/* TOT - Total Octets Transmitted */
	uint32_t dc;	/* DC - Defer Count */
	uint32_t res8;
	uint32_t plt64c; /* PLT64C - Packet Xmit Less than 64 Bytes Count */
	uint32_t res9[7];
	uint32_t tsctc;	/* TSCTC - TCP Segmentation Context Transmitted Count */
	uint32_t res10;
	uint32_t tsctfc; /* TSCTFC - TCP Segmentation Context Xmit Fail Count */
	uint32_t res11;
	uint32_t ibic;	/* IBIC - Illegal Byte During Idle Stream Count */
	uint32_t res12;
	uint32_t rfc;	/* RFC - Remote Fault Count */
	uint32_t res13;
	uint32_t lfc;	/* LFC - Local Fault Count */
	uint32_t res14;
	uint32_t pfrc;	/* PFRC - Pause Frame Receive Count */
	uint32_t res15;
	uint32_t pftc;	/* PFTC - Pause Frame Transmit Count */
	uint32_t res16;
	uint32_t mcfrc;	/* MCFRC - MAC Control Frame Receive Count */
	uint32_t res17;
	uint32_t mcftc;	/* MCFTC - MAC Control Frame Transmit Count */
	uint32_t res18;
	uint32_t xonrxc;	/* XONRXC - XON Received Count */
	uint32_t res19;
	uint32_t xontxc;	/* XONTXC - XON Transmitted Count */
	uint32_t res20;
	uint32_t xoffrxc;	/* XOFFRXC - XOFF Received Count */
	uint32_t res21;
	uint32_t xofftxc;	/* XOFFTXC - XOFF Transmitted Count */
	uint32_t res22;
	uint32_t rjc;	/* RJC - Receive Jabber Count */
	uint32_t res23;
} hw_stats_t, *p_hw_stats_t;


/* ======================================= */

/* DIAGNOSTIC REGISTERS */

/* TREG - Test Register (MAC) */
typedef union _treg_t {
	uint32_t val;
	struct {
#ifdef _BIT_FIELDS_HTOL
		uint32_t res2:9;
		uint32_t busy:8;
		uint32_t res1:4;
		uint32_t trfd:1;
		uint32_t res0:10;
#else
		uint32_t res0:10;
		uint32_t trfd:1;
		uint32_t res1:4;
		uint32_t busy:8;
		uint32_t res2:9;
#endif
	} bits;
} treg_t, *p_treg_t;


typedef struct _hw_diag_t {
	uint32_t rdfh;		/* Receive Data FIFO Head */
	uint32_t res1;
	uint32_t rdft;		/* Receive Data FIFO Tail */
	uint32_t res2[3];
	uint32_t rdfts;		/* Receive Data FIFO Tail Saved */
	uint32_t res3;
	uint32_t rdfpc;		/* Receive Data FIFO Packet Count */
	uint32_t res4;
	uint32_t tdfh;		/* Transmit Data FIFO Head */
	uint32_t res5;
	uint32_t tdft;		/* Transmit Data FIFO Tail */
	uint32_t res6[3];
	uint32_t tdfts;		/* Transmit Data FIFO Tail Saved */
	uint32_t res7;
	uint32_t tdfpc;		/* Transmit Data FIFO Packet Count */
	uint32_t res8;
	treg_t   treg;		/* Test Register */
	uint32_t res9;
	uint32_t rpr;		/* Receive Page Register */
	uint32_t res10;
	uint32_t tpr;		/* Transmit Page Register */
	uint32_t res11[999];
	uint32_t rpdbm[1024];	/* Receive Packet or Descriptor Buffer Memory */
	uint32_t tpdbm[1024];	/* Transmit Packet or Descriptor Buffer Mem */
} hw_diag_t, *p_hw_diag_t;

/* ======================================= */


/* General Registers - 0x0 */
typedef struct _global_t {
	ctrl0_t ctrl0;
	uint32_t res0;
	ctrl1_t ctrl1;
	uint32_t res1;
	status_t status;
	uint32_t res2;
	eecd_t eecd;
	uint32_t res3;
	mfs_t mfs;
	uint32_t res4[23];

	/* Interrupt Registers - 0x80 */
	intr_t icr;
	uint32_t res5;
	intr_t ics;
	uint32_t res6;
	intr_t ims;
	uint32_t res7;
	intr_t imc;
} global_t, *p_global_t;

/* Receive Registers - 0x100 */
typedef struct _rx_t {
	rxctl_t rctl;
	uint32_t res0;
	fcrt_t fcrtl;
	uint32_t res1;
	fcrt_t fcrth;
	uint32_t res2;
	uint64_t db;
	uint32_t rdlen;
	uint32_t res3;
	uint32_t rdh;
	uint32_t res4;
	uint32_t rdt;
	uint32_t res5;
	rdtr_t rdtr;
	uint32_t res6;
	rxdctl_t rxdctl;
	uint32_t res7;
	raidc_t raidc;
	uint32_t res8[3];
	rxcsum_t rxcsum;
	uint32_t res9[9];
	mac_addr_t mac[IXGE_MAC_ENTRIES];
	uint32_t mta[128];
	uint32_t vfta[128];
} rx_t, *p_rx_t;

/* Transmit Registers - 0x600 */
typedef struct _tx_t {
	tctl_t tctl;
	uint32_t res0;
	uint64_t db;
	uint32_t tdlen;
	uint32_t res1;
	uint32_t tdh;
	uint32_t res2;
	uint32_t tdt;
	uint32_t res3;
	tidv_t tidv;
	uint32_t res4;
	txdctl_t txdctl;
	uint32_t res5;
	tspmt_t tspmt;
	uint32_t res6;
	pap_t pap;
} tx_t, *p_tx_t;

/* Physical Registers  - 0x700 */
typedef struct _phys_t {
	pcsc_t	pcsc;
	uint32_t res0;
	pcsc2_t	pcsc2;
	uint32_t res1;
	pcss_t	pcss;
	uint32_t res2;
	pcss2_t	pcss2;
	uint32_t res3;
	xpcss_t	xpcss;
	uint32_t res4;
	uint32_t uccr;
	uint32_t res5;
	xpcstc_t xpcstc;
	uint32_t res6;
	maca_t	maca;
	uint32_t res7;
	uint32_t apae;
	uint32_t res8;
	uint32_t ard;
	uint32_t res9;
	uint32_t ais;
	uint32_t res10;
	msca_t	msca;
	uint32_t res11;
	msrwd_t	msrwd;
} phys_t, *p_phys_t;

/* ======================================= */

/* Entire register map for IXGE */
typedef struct _ixge_regs_t {
	global_t global;		/* General and Intr Regs */
	uint32_t res0[25];
	rx_t rx;			/* Receive Registers */
	tx_t tx;			/* Transmit Registers */
	uint32_t res1[47];
	phys_t	phys;			/* Physical Registers */
	uint32_t res2[41];
	uint32_t res3[1534];		/* Wakeup Registers - Not defined */
	hw_stats_t stats;		/* Statistics Registers */
	uint32_t res4[1928];
	hw_diag_t diag;			/* Diagnostics Registers */
} ixge_regs_t, *p_ixge_regs_t;

/* ======================================= */

#define	IXGE_DESC_TYPE_CONTEXT		0x0
#define	IXGE_DESC_TYPE_DATA		0x1

/* Transmit Descriptor - Data Descriptor Format */
typedef union _tdesc_t {
	uint64_t val;
	struct {
#ifdef _BIG_ENDIAN
		struct {
#ifdef _BIT_FIELDS_HTOL
			uint32_t pri:3;
			uint32_t cfi:1;
			uint32_t vlan:12;	/* Special */
			uint32_t res2:6;
			uint32_t txsm:1;
			uint32_t ixsm:1;	/* POTPS */
			uint32_t res1:4;	/* Rsv */
			uint32_t res0:3;
			uint32_t dd:1;		/* STA */

#else
			uint32_t dd:1;		/* STA */
			uint32_t res0:3;
			uint32_t res1:4;	/* Rsv */
			uint32_t ixsm:1;	/* POTPS */
			uint32_t txsm:1;
			uint32_t res2:6;
			uint32_t vlan:12;	/* Special */
			uint32_t cfi:1;
			uint32_t pri:3;
#endif
		} hw;
#endif
		struct {
#ifdef _BIT_FIELDS_HTOL
			uint32_t ide:1;
			uint32_t vle:1;
			uint32_t res1:2;
			uint32_t rs:1;
			uint32_t tse:1;
			uint32_t res0:1;
			uint32_t eop:1;		/* DCMD */
			uint32_t dtyp:4;	/* DTYP */
			uint32_t dtalen:20;	/* DTALEN */

#else
			uint32_t dtalen:20;	/* DTALEN */
			uint32_t dtyp:4;	/* DTYP */
			uint32_t eop:1;		/* DCMD */
			uint32_t res0:1;
			uint32_t tse:1;
			uint32_t rs:1;
			uint32_t res1:2;
			uint32_t vle:1;
			uint32_t ide:1;
#endif
		} lw;
#ifndef _BIG_ENDIAN
		struct {
#ifdef _BIT_FIELDS_HTOL
			uint32_t pri:3;
			uint32_t cfi:1;
			uint32_t vlan:12;	/* Special */
			uint32_t res2:6;
			uint32_t txsm:1;
			uint32_t ixsm:1;	/* POTPS */
			uint32_t res1:4;	/* Rsv */
			uint32_t res0:3;
			uint32_t dd:1;		/* STA */

#else
			uint32_t dd:1;		/* STA */
			uint32_t res0:3;
			uint32_t res1:4;	/* Rsv */
			uint32_t ixsm:1;	/* POTPS */
			uint32_t txsm:1;
			uint32_t res2:6;
			uint32_t vlan:12;	/* Special */
			uint32_t cfi:1;
			uint32_t pri:3;
#endif
		} hw;
#endif
	} bits;
} tdesc_t, *p_tdesc_t;

/* Transmit Descriptor - Context Descriptor Format (1st 64 bits) */
typedef union _tip_desc0_t {
	uint64_t val;
	struct {
#ifdef _BIG_ENDIAN
		struct {
#ifdef _BIT_FIELDS_HTOL
			uint32_t tucse:16;
			uint32_t tucso:8;
			uint32_t tucss:8;
#else
			uint32_t tucss:8;
			uint32_t tucso:8;
			uint32_t tucse:16;
#endif
		} hw;
#endif
		struct {
#ifdef _BIT_FIELDS_HTOL
			uint32_t ipcse:16;
			uint32_t ipcso:8;
			uint32_t ipcss:8;
#else
			uint32_t ipcss:8;
			uint32_t ipcso:8;
			uint32_t ipcse:16;
#endif
		} lw;
#ifndef _BIG_ENDIAN
		struct {
#ifdef _BIT_FIELDS_HTOL
			uint32_t tucse:16;
			uint32_t tucso:8;
			uint32_t tucss:8;
#else
			uint32_t tucss:8;
			uint32_t tucso:8;
			uint32_t tucse:16;
#endif
		} hw;
#endif
	} bits;
} tip_desc0_t, *p_tip_desc0_t;

/* Transmit Descriptor - Context Descriptor Format (2nd 64 bits) */
typedef union _tip_desc1_t {
	uint64_t val;
	struct {
#ifdef _BIG_ENDIAN
		struct {
#ifdef _BIT_FIELDS_HTOL
			uint32_t mss:16;	/* MSS */
			uint32_t hdrlen:8;	/* HDRLEN */
			uint32_t res1:4;	/* Rsv */
			uint32_t res0:3;	/* STA */
			uint32_t dd:1;
#else
			uint32_t dd:1;
			uint32_t res0:3;	/* STA */
			uint32_t res1:4;	/* Rsv */
			uint32_t hdrlen:8;	/* HDRLEN */
			uint32_t mss:16;	/* MSS */
#endif
		} hw;
#endif
		struct {
#ifdef _BIT_FIELDS_HTOL
			uint32_t ide:1;		/* TUCMD */
			uint32_t res0:3;
			uint32_t rs:1;
			uint32_t tse:1;
			uint32_t ip:1;
			uint32_t tcp:1;
			uint32_t dtyp:4;	/* DTYP */
			uint32_t paylen:20;	/* PAYLEN */

#else
			uint32_t paylen:20;	/* PAYLEN */
			uint32_t dtyp:4;	/* DTYP */
			uint32_t tcp:1;
			uint32_t ip:1;
			uint32_t tse:1;
			uint32_t rs:1;
			uint32_t res0:3;
			uint32_t ide:1;		/* TUCMD */
#endif
		} lw;
#ifndef _BIG_ENDIAN
		struct {
#ifdef _BIT_FIELDS_HTOL
			uint32_t mss:16;	/* MSS */
			uint32_t hdrlen:8;	/* HDRLEN */
			uint32_t res1:4;	/* Rsv */
			uint32_t res0:3;	/* STA */
			uint32_t dd:1;
#else
			uint32_t dd:1;
			uint32_t res0:3;	/* STA */
			uint32_t res1:4;	/* Rsv */
			uint32_t hdrlen:8;	/* HDRLEN */
			uint32_t mss:16;	/* MSS */
#endif
		} hw;
#endif
	} bits;
} tip_desc1_t, *p_tip_desc1_t;

typedef struct _tx_desc_t {
	uint64_t addr;
	tdesc_t cmd;
} tx_desc_t, *p_tx_desc_t;

typedef struct _tx_ip_desc_t {
	tip_desc0_t desc0;
	tip_desc1_t desc1;
} tx_ip_desc_t, *p_tx_ip_desc_t;

/* ======================================= */

/* Receive Descriptor */
typedef union _rdesc_t {
	uint64_t val;
	struct {
#ifdef _BIG_ENDIAN
		struct {
#ifdef _BIT_FIELDS_HTOL
			uint32_t pri:3;		/* SPECIAL */
			uint32_t cfi:1;
			uint32_t vlan:12;
			uint32_t rxe:1;		/* ERRORS */
			uint32_t ipe:1;
			uint32_t tcpe:1;
			uint32_t res2:1;
			uint32_t p:1;
			uint32_t res1:1;
			uint32_t se:1;
			uint32_t ce:1;
			uint32_t pif:1;		/* STATUS */
			uint32_t ipcs:1;
			uint32_t tcpcs:1;
			uint32_t res0:1;
			uint32_t vp:1;
			uint32_t ixsm:1;
			uint32_t eop:1;
			uint32_t dd:1;
#else
			uint32_t dd:1;
			uint32_t eop:1;
			uint32_t ixsm:1;
			uint32_t vp:1;
			uint32_t res0:1;
			uint32_t tcpcs:1;
			uint32_t ipcs:1;
			uint32_t pif:1;		/* STATUS */
			uint32_t ce:1;
			uint32_t se:1;
			uint32_t res1:1;
			uint32_t p:1;
			uint32_t res2:1;
			uint32_t tcpe:1;
			uint32_t ipe:1;
			uint32_t rxe:1;		/* ERRORS */
			uint32_t vlan:12;
			uint32_t cfi:1;
			uint32_t pri:3;		/* SPECIAL */
#endif
		} hw;
#endif
		struct {
#ifdef _BIT_FIELDS_HTOL
			uint32_t cksum:16;
			uint32_t length:16;
#else
			uint32_t length:16;
			uint32_t cksum:16;
#endif
		} lw;
#ifndef _BIG_ENDIAN
		struct {
#ifdef _BIT_FIELDS_HTOL
			uint32_t pri:3;		/* SPECIAL */
			uint32_t cfi:1;
			uint32_t vlan:12;
			uint32_t rxe:1;		/* ERRORS */
			uint32_t ipe:1;
			uint32_t tcpe:1;
			uint32_t res2:1;
			uint32_t p:1;
			uint32_t res1:1;
			uint32_t se:1;
			uint32_t ce:1;
			uint32_t pif:1;		/* STATUS */
			uint32_t ipcs:1;
			uint32_t tcpcs:1;
			uint32_t res0:1;
			uint32_t vp:1;
			uint32_t ixsm:1;
			uint32_t eop:1;
			uint32_t dd:1;
#else
			uint32_t dd:1;
			uint32_t eop:1;
			uint32_t ixsm:1;
			uint32_t vp:1;
			uint32_t res0:1;
			uint32_t tcpcs:1;
			uint32_t ipcs:1;
			uint32_t pif:1;		/* STATUS */
			uint32_t ce:1;
			uint32_t se:1;
			uint32_t res1:1;
			uint32_t p:1;
			uint32_t res2:1;
			uint32_t tcpe:1;
			uint32_t ipe:1;
			uint32_t rxe:1;		/* ERRORS */
			uint32_t vlan:12;
			uint32_t cfi:1;
			uint32_t pri:3;		/* SPECIAL */
#endif
		} hw;
#endif
	} bits;
} rdesc_t, *p_rdesc_t;

typedef struct _rx_desc_t {
	uint64_t	addr;
	rdesc_t		cmd;
} rx_desc_t, *p_rx_desc_t;


/*
 * Restore to default packing.
 */
#pragma pack()

#ifdef _KERNEL
typedef struct _dev_regs_t {
	ddi_acc_handle_t	ixge_regh;
	p_ixge_regs_t		ixge_regp;
	unsigned char		*ixge_romp;	/* XXXPAS EEPROM not fcode */
	ddi_acc_handle_t	ixge_romh;	/* rom handle */
	ddi_acc_handle_t	ixge_pciregh;
	p_pci_cfg_t		ixge_pciregp;
} dev_regs_t, *p_dev_regs_t;

/*
 * Soft copy of key device registers, this will help avoid
 * pio's to access the device to get it's configuration
 * state.
 */
typedef struct _soft_regs_t {
	uint32_t intr_mask;     /* Soft copy of interrupt mask */
	uint32_t etx_cfg;
	uint32_t erx_cfg;
	uint32_t tx_mac_cfg;
	uint32_t rx_mac_cfg;
	uint32_t mif_cfg;
	uint32_t mif_int_mask;
	mblk_t *filter_msg;
} soft_regs_t, *p_soft_regs_t;

#define FILTER_M_CTL 0xDCEF1
#define HASH_BITS 8
#define NMCFILTER_BITS 4096
#define HASH_REG_WIDTH 32
#define BROADCAST_HASH_WORD 0x0f
#define BROADCAST_HASH_BIT 0x8000
#endif

#ifdef	__cplusplus
}
#endif

#endif
